#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <curl/curl.h>
#include "signer.h"

static size_t
WriteMemoryCallback(void* contents, size_t size, size_t nmemb, void* userp)
{
    printf("%.*s", size * nmemb, (char*)contents);
    return size * nmemb;
}

int main(void)
{
    //initialize signature params
    RequestParams* request = new RequestParams("POST", "30030113-3657-4fb6-a7ef-90764239b038.apigw.cn-north-1.huaweicloud.com", "/app1",
        "Action=ListUsers&Version=2010-05-08", "demo");
    request->addHeader("content-type", "application/x-www-form-urlencoded; charset=utf-8");
    // 认证用的ak和sk硬编码到代码中或者明文存储都有很大的安全风险，建议在配置文件或者环境变量中密文存放，使用时解密，确保安全；
    // 本示例以ak和sk保存在环境变量中为例，运行本示例前请先在本地环境中设置环境变量HUAWEICLOUD_SDK_AK和HUAWEICLOUD_SDK_SK。
    Signer signer(getenv("HUAWEICLOUD_SDK_AK"), getenv("HUAWEICLOUD_SDK_SK"));

    //generate signature. generated Authorization header is inserted into params.headers
    signer.createSignature(request);

    //send http request using curl library
    curl_global_init(CURL_GLOBAL_ALL);
    CURL *curl = curl_easy_init();
    CURLcode res;

    curl_easy_setopt(curl, CURLOPT_CUSTOMREQUEST, request->getMethod().c_str());
    std::string url = "http://" + request->getHost() + request->getUri() + "?" + request->getQueryParams();
    curl_easy_setopt(curl, CURLOPT_URL, url.c_str());
    struct curl_slist *chunk = NULL;
    std::set<Header>::iterator it;
    for (auto header : *request->getHeaders()) {
        std::string headerEntry = header.getKey() + ": " + header.getValue();
        printf("%s\n", headerEntry.c_str());
        chunk = curl_slist_append(chunk, headerEntry.c_str());
    }
    printf("-------------\n");
    curl_easy_setopt(curl, CURLOPT_HTTPHEADER, chunk);
    curl_easy_setopt(curl, CURLOPT_COPYPOSTFIELDS, request->getPayload().c_str());
    curl_easy_setopt(curl, CURLOPT_NOBODY, 0L);
    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
    //curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
    res = curl_easy_perform(curl);
    if (res != CURLE_OK) {
        fprintf(stderr, "curl_easy_perform() failed: %s\n", curl_easy_strerror(res));
    }
    else {
        long status;
        curl_easy_getinfo(curl, CURLINFO_HTTP_CODE, &status);
        printf("\nstatus %d\n", status);
    }
    curl_easy_cleanup(curl);

    curl_global_cleanup();

    return 0;
}

